import { useReducer, useEffect, useRef } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var reducer = function (state, action) {
    switch (action.type) {
        case 'error':
            return __assign({}, state, { error: action.error, loading: false });
        case 'reset':
            return __assign({}, state, { value: action.defaultValue });
        case 'value':
            return __assign({}, state, { loading: false, value: action.value });
        default:
            return state;
    }
};
var useLoadingValue = (function (defaultValue) {
    var _a = useReducer(reducer, {
        loading: defaultValue === undefined || defaultValue === null,
        value: defaultValue,
    }), state = _a[0], dispatch = _a[1];
    var reset = function () {
        dispatch({ type: 'reset', defaultValue: defaultValue });
    };
    var setError = function (error) {
        dispatch({ type: 'error', error: error });
    };
    var setValue = function (value) {
        dispatch({ type: 'value', value: value });
    };
    return {
        error: state.error,
        loading: state.loading,
        reset: reset,
        setError: setError,
        setValue: setValue,
        value: state.value,
    };
});

var useComparatorRef = function (value, isEqual, onChange) {
    var ref = useRef(value);
    useEffect(function () {
        if (!isEqual(value, ref.current)) {
            ref.current = value;
            if (onChange) {
                onChange();
            }
        }
    });
    return ref;
};
var useIsEqualRef = function (value, onChange) {
    return useComparatorRef(value, function (v1, v2) { return v1.isEqual(v2); }, onChange);
};

var useCollection = (function (query, options) {
    var _a = useLoadingValue(), error = _a.error, loading = _a.loading, reset = _a.reset, setError = _a.setError, setValue = _a.setValue, value = _a.value;
    var ref = useIsEqualRef(query, reset);
    useEffect(function () {
        var listener = options
            ? ref.current.onSnapshot(options, setValue, setError)
            : ref.current.onSnapshot(setValue, setError);
        return function () {
            listener();
        };
    }, [ref.current]);
    return {
        error: error,
        loading: loading,
        value: value,
    };
});

var useDocument = (function (docRef, options) {
    var _a = useLoadingValue(), error = _a.error, loading = _a.loading, reset = _a.reset, setError = _a.setError, setValue = _a.setValue, value = _a.value;
    var ref = useIsEqualRef(docRef, reset);
    useEffect(function () {
        var listener = options
            ? ref.current.onSnapshot(options, setValue, setError)
            : ref.current.onSnapshot(setValue, setError);
        return function () {
            listener();
        };
    }, [ref.current]);
    return {
        error: error,
        loading: loading,
        value: value,
    };
});

export { useCollection, useDocument };
